@@ -66,7 +66,7 @@ class DiscordUser(immp.User):
     """
 
     @classmethod
-    def from_user(cls, discord, user):
+    def from_user(cls, discord_, user):
         """
         Convert a :class:`discord.User` into a :class:`.User`.
 
@@ -85,7 +85,7 @@ class DiscordUser(immp.User):
         # Avatar URL is an Asset object, URL only available via __str__.
         avatar = str(user.avatar_url) if user.avatar_url else None
         return cls(id_=user.id,
-                   plug=discord,
+                   plug=discord_,
                    username=username,
                    real_name=real_name,
                    avatar=avatar,
@@ -102,21 +102,22 @@ class DiscordRichText(immp.RichText):
     # 3) Formatting characters may be escaped with a backslash.
     # This still isn't perfect, but provides a good approximation outside of edge cases.
     _outside_chars = r"0-9a-z*_~"
-    _tag_chars = r"```|\*\*|__|~~|[*_`]"
+    _tag_chars = r"\*\*|__|~~|[*_`]"
     _inside_chars = r"\s\1"
     _format_regex = re.compile(r"(?<![{0}\\])({1})(?![{2}])(.+?)(?<![{2}\\])\1(?![{0}])"
                                .format(_outside_chars, _tag_chars, _inside_chars))
+    _pre_regex = re.compile(r"```\n?(.+?)\n?```", re.DOTALL)
 
     _mention_regex = re.compile(r"<@!?(\d+)>")
     _channel_regex = re.compile(r"<#(\d+)>")
     _emoji_regex = re.compile(r"<(:[^: ]+?:)\d+>")
 
     @classmethod
-    def _sub_channel(cls, discord, match):
-        return "#{}".format(discord._client.get_channel(int(match.group(1))).name)
+    def _sub_channel(cls, discord_, match):
+        return "#{}".format(discord_._client.get_channel(int(match.group(1))).name)
 
     @classmethod
-    def from_markdown(cls, discord, text):
+    def from_markdown(cls, discord_, text, channel=None):
         """
         Convert a string of Markdown into a :class:`.RichText`.
 
@@ -125,40 +126,65 @@ class DiscordRichText(immp.RichText):
                 Related plug instance that provides the text.
             text (str):
                 Markdown formatted text.
+            channel (.Channel):
+                Related channel, used to retrieve mentioned users as members.
 
         Returns:
             .DiscordRichText:
                 Parsed rich text container.
         """
         changes = defaultdict(dict)
-        while True:
-            match = cls._format_regex.search(text)
-            if not match:
-                break
-            start = match.start()
-            end = match.end()
-            tag = match.group(1)
-            # Strip the tag characters from the message.
-            text = text[:start] + match.group(2) + text[end:]
-            end -= 2 * len(tag)
-            # Record the range where the format is applied.
-            field = cls.tags[tag]
-            changes[start][field] = True
-            changes[end][field] = False
-            # Shift any future tags back.
-            for pos in sorted(changes):
-                if pos > end:
-                    changes[pos - 2 * len(tag)].update(changes.pop(pos))
-        for match in cls._mention_regex.finditer(text):
-            user = discord._client.get_user(int(match.group(1)))
+        plain = ""
+        done = False
+        while not done:
+            # Identify pre blocks, parse formatting only outside of them.
+            match = cls._pre_regex.search(text)
+            if match:
+                parse = text[:match.start()]
+                pre = match.group(1)
+                text = text[match.end():]
+            else:
+                parse = text
+                done = True
+            offset = len(plain)
+            while True:
+                match = cls._format_regex.search(parse)
+                if not match:
+                    break
+                start = match.start()
+                end = match.end()
+                tag = match.group(1)
+                # Strip the tag characters from the message.
+                parse = parse[:start] + match.group(2) + parse[end:]
+                end -= 2 * len(tag)
+                # Record the range where the format is applied.
+                field = cls.tags[tag]
+                changes[offset + start][field] = True
+                changes[offset + end][field] = False
+                # Shift any future tags back.
+                for pos in sorted(changes):
+                    if pos > offset + end:
+                        changes[pos - 2 * len(tag)].update(changes.pop(pos))
+            plain += parse
+            if not done:
+                changes[len(plain)]["pre"] = True
+                changes[len(plain + pre)]["pre"] = False
+                plain += pre
+        dc_channel = discord_._get_channel(channel)
+        if isinstance(dc_channel, discord.TextChannel):
+            getter = dc_channel.guild.get_member
+        else:
+            getter = discord_._client.get_user
+        for match in cls._mention_regex.finditer(plain):
+            user = getter(int(match.group(1)))
             if user:
-                changes[match.start()]["mention"] = DiscordUser.from_user(discord, user)
+                changes[match.start()]["mention"] = DiscordUser.from_user(discord_, user)
                 changes[match.end()]["mention"] = None
         segments = []
         points = list(sorted(changes.keys()))
         formatting = {}
         # Iterate through text in change start/end pairs.
-        for start, end in zip([0] + points, points + [len(text)]):
+        for start, end in zip([0] + points, points + [len(plain)]):
             formatting.update(changes[start])
             if start == end:
                 # Zero-length segment at the start or end, ignore it.
@@ -167,16 +193,16 @@ class DiscordRichText(immp.RichText):
                 user = formatting["mention"]
                 part = "@{}".format(user.username or user.real_name)
             else:
-                part = emojize(text[start:end], use_aliases=True)
+                part = emojize(plain[start:end], use_aliases=True)
                 # Strip Discord channel/emoji tags, replace with a plain text representation.
-                part = cls._channel_regex.sub(partial(cls._sub_channel, discord), part)
+                part = cls._channel_regex.sub(partial(cls._sub_channel, discord_), part)
                 part = cls._emoji_regex.sub(r"\1", part)
             segments.append(immp.Segment(part, **formatting))
         return cls(segments)
 
     @classmethod
-    def _sub_emoji(cls, discord, match):
-        for emoji in discord._client.emojis:
+    def _sub_emoji(cls, discord_, match):
+        for emoji in discord_._client.emojis:
             if emoji.name == match.group(1):
                 return str(emoji)
         return ":{}:".format(match.group(1))
@@ -211,6 +237,9 @@ class DiscordRichText(immp.RichText):
                 # Add any new tags that start at this segment.
                 if getattr(segment, attr) and tag not in active:
                     text += tag
+                    if tag == "```":
+                        # First line of pre block would set the code language and be hidden.
+                        text += "\n"
                     active.append(tag)
             if segment.mention and isinstance(segment.mention.plug, DiscordPlug):
                 text += "<@{}>".format(segment.mention.id)
@@ -267,7 +296,7 @@ class DiscordMessage(immp.Message):
     """
 
     @classmethod
-    def from_message(cls, discord, message, edited=False, deleted=False):
+    def from_message(cls, discord_, message, edited=False, deleted=False):
         """
         Convert a :class:`discord.Message` into a :class:`.Message`.
 
@@ -286,10 +315,11 @@ class DiscordMessage(immp.Message):
                 Parsed message object.
         """
         text = None
-        user = DiscordUser.from_user(discord, message.author)
+        channel = immp.Channel(discord_, message.channel.id)
+        user = DiscordUser.from_user(discord_, message.author)
         attachments = []
         if message.content:
-            text = DiscordRichText.from_markdown(discord, message.content)
+            text = DiscordRichText.from_markdown(discord_, message.content, channel)
         for attach in message.attachments:
             if attach.filename.endswith((".jpg", ".png", ".gif")):
                 type_ = immp.File.Type.image
@@ -305,7 +335,7 @@ class DiscordMessage(immp.Message):
                 attachments.append(immp.File(type_=immp.File.Type.image,
                                              source=embed.image.url))
         return immp.SentMessage(id_=message.id,
-                                channel=immp.Channel(discord, message.channel.id),
+                                channel=channel,
                                 # Timestamps are naive but in UTC.
                                 at=message.created_at.replace(tzinfo=timezone.utc),
                                 # Edited timestamp is blank for new messages, but updated in
